--
-- Unidad III
--

use Northwind;

--
-- Consultas básicas
--

--
-- Consultas básicas - Select (1)
--

select ProductID, ProductName, QuantityPerUnit
from Products;

--
-- Consultas básicas - Select (2)
--

select ProductName, QuantityPerUnit
from Products
where ProductID = 10;

--
-- Consultas básicas - Select (3)
--

select ProductID, ProductName, QuantityPerUnit
from Products
where QuantityPerUnit like '%boxes%';

--
-- Consultas básicas - Select (4)
--

select ProductID, ProductName, QuantityPerUnit
from Products
where (QuantityPerUnit like '%boxes%') or (QuantityPerUnit like '%bottles%');

--
-- Consultas básicas - Select (5)
--

select ProductID, ProductName, QuantityPerUnit
from Products
where ProductID between 1 and 10;

--
-- Consultas básicas - Select (6)
--

select LastName, FirstName, City
from Employees
where City in ('London', 'Seattle');

--
-- Consultas básicas - Select (7)
--

select LastName, FirstName, Region
from Employees
where Region is not null;

--
-- Consultas básicas - Orden de datos
--

select ProductID, ProductName, UnitPrice
from Products
order by UnitPrice desc;

--
-- Consultas básicas - Eliminación de duplicados
--

select distinct CustomerID
from Orders
order by CustomerID;

select CustomerID
from Orders
order by CustomerID;

--
-- Consultas básicas - Cambio de nombres a las columnas
--

select LastName 'Apellido', FirstName as 'Nombre'
from Employees;

--
-- Consultas básicas - Uso de constantes
--

select LastName 'Apellido', FirstName as 'Nombre', 'ID:'
from Employees;

--
-- Consultas básicas - CASE simple
--

select LastName, FirstName, 
	case TitleOfCourtesy
		when 'Mrs.' then 'Señora'
        when 'Mr.' then 'Señor'
        when 'Ms.' then 'Señorita'
        -- else 'Otro'
	end as 'Titulo'
from Employees;

--
-- Consultas básicas - CASE búsqueda
--

select ProductID, ProductName,
	case 
		when UnitPrice > 0 and UnitPrice < 30 then 'Hasta 30'
        when UnitPrice >= 30 and UnitPrice < 60 then 'Entre $30 y $60'
        else 'Más de $60'
	end as 'UnitPrice'
from Products;

--
-- Consultas básicas - Coalesce
--

select CustomerID, Region, Fax, coalesce(Region, Fax) as 'Not null'
from Customers;

--
-- Consultas básicas - ifnull
--

select CustomerID, ifnull(Region, '-') as 'Region'
from Customers;

-- también se puede usar coalesce

select CustomerID, coalesce(Region, '-') as 'Region'
from Customers;

--
-- Consultas básicas - nullif
--

create table if not exists Budgets (  
    dept int auto_increment not null,  
    cur_year decimal null,  
    prev_year decimal null,
    primary key (dept)
);

insert into Budgets (cur_year, prev_year) values (100000, 150000);
insert into Budgets (cur_year, prev_year) values (null, 300000);
insert into Budgets (cur_year, prev_year) values (0, 100000);
insert into Budgets (cur_year, prev_year) values (null, 150000);
insert into Budgets (cur_year, prev_year) values (300000, 250000);

select cur_year, prev_year
from Budgets;

select cur_year, prev_year, coalesce(cur_year, prev_year) as ‘Col1’
from Budgets;

select cur_year, prev_year, coalesce(cur_year, prev_year) as ‘Col1’, nullif(coalesce(cur_year, prev_year), 0.00) as ‘Col2’
from Budgets;

select avg(nullif(coalesce(cur_year, prev_year), 0.00)) as ‘Promedio’
from Budgets;

drop table if exists Budgets;

--
-- Consultas básicas - Inserción de filas con valores por defecto
--

create table if not exists Budgets (  
    dept int auto_increment not null,  
    cur_year decimal null default 0,  
    prev_year decimal null default 0,
    primary key (dept)
);

insert into Budgets (cur_year, prev_year) values (100000, 150000);
insert into Budgets (cur_year, prev_year) values (null, 300000);
insert into Budgets (cur_year, prev_year) values (0, 100000);
insert into Budgets (cur_year, prev_year) values (null, 150000);
insert into Budgets (cur_year, prev_year) values (300000, 250000);
-- valor por defecto en una columna
insert into Budgets (prev_year) values (250000);

-- valor por defecto en todas las columnas
insert into Budgets values ();

select * from Budgets;

drop table if exists Budgets;

--
-- Consultas básicas - Borrado de filas
--

create table if not exists Budgets (  
    dept int auto_increment not null,  
    cur_year decimal null,  
    prev_year decimal null,
    primary key (dept)
);

insert into Budgets (cur_year, prev_year) values (100000, 150000);
insert into Budgets (cur_year, prev_year) values (null, 300000);
insert into Budgets (cur_year, prev_year) values (0, 100000);
insert into Budgets (cur_year, prev_year) values (null, 150000);
insert into Budgets (cur_year, prev_year) values (300000, 250000);

select * from Budgets;

delete from Budgets
where dept = 5;

truncate Budgets;

drop table if exists Budgets;

--
-- Consultas básicas - Modificación de filas
--

create table if not exists Budgets (  
    dept int auto_increment not null,  
    cur_year decimal null,  
    prev_year decimal null,
    primary key (dept)
);

insert into Budgets (cur_year, prev_year) values (100000, 150000);
insert into Budgets (cur_year, prev_year) values (null, 300000);
insert into Budgets (cur_year, prev_year) values (0, 100000);
insert into Budgets (cur_year, prev_year) values (null, 150000);
insert into Budgets (cur_year, prev_year) values (300000, 250000);

select * from Budgets;

update Budgets
set prev_year = 950000 where dept = 1;

drop table if exists Budgets;

--
-- Múltiples tablas
--




--
-- Múltiples tablas - Inner join
--

select ProductID, ProductName, CategoryName
from Products join Categories
on Products.CategoryID = Categories.CategoryID
where ProductID between 1 and 24;

--
-- Múltiples tablas - Outer join
--

select Customers.CustomerID, OrderID, OrderDate
from Customers left join Orders
on Customers.CustomerID = Orders.CustomerID;

--
-- Múltiples tablas - Cross join
--

select Employees.EmployeeID, OrderID
from Employees cross join Orders
order by Employees.EmployeeID, OrderID;

select count(EmployeeID) from Employees;
select count(OrderID) from Orders;

--
-- Múltiples tablas - Combinación de más de 2 tablas
--

select ProductID, ProductName, CompanyName, CategoryName
from Products join Suppliers
on Products.SupplierID = Suppliers.SupplierID
join Categories
on Products.CategoryID = Categories.CategoryID;

--
-- Múltiples tablas - Join de una tabla con sí misma
-- 

select E1.EmployeeID, E1.LastName, E1.FirstName, E1.ReportsTo, E2.LastName, E2.FirstName
from Employees E1 left join Employees E2
on E1.ReportsTo = E2.EmployeeID
order by E1.EmployeeID;

select EmployeeID, LastName, FirstName, ReportsTo
from Employees;


--
-- Múltiples tablas - Combinación de varios ResultSets
-- 

select CompanyName, City, PostalCode 
from Customers
union all
select concat(LastName, ' ', FirstName), City, PostalCode 
from Employees;

--
-- Múltiples tablas - Creación de una tabla desde un ResultSet
-- 

create table PriceTable
select ProductName, UnitPrice
from Products;

select * from PriceTable;

drop table if exists PriceTable;

create table PriceTable like Products;

drop table if exists PriceTable;

--
-- Múltiples tablas - Creación de una tabla temporal
-- 

create temporary table PriceTable
select ProductName, UnitPrice
from Products;

select * from PriceTable;

drop table if exists PriceTable;

--
-- Técnicas avanzadas - Subconsultas
-- 

select LastName, FirstName
from Employees E
where exists (select * from Orders where E.EmployeeID = Orders.EmployeeID);

--
-- Técnicas avanzadas - Inserción de filas basadas en otras tablas
-- 

create table Customers1 like Customers;

insert into Customers1
select concat(substring(FirstName, 1, 3), substring(LastName, 1, 2)), LastName, FirstName, Title, null, null, null, null, null, null, null
from Employees;

select * from Customers1;

drop table Customers1;

--
-- Técnicas avanzadas - Borrado de filas basadas en otras tablas (subconsulta)
-- 

delete from `Order Details`
where OrderID in
       (select OrderID from Orders where OrderDate = '2017/4/14');


--
-- Técnicas avanzadas - Modificación de filas basadas en otras tablas (subconsultas)
-- 

update Products 
set UnitPrice = UnitPrice + 2
where SupplierID in (select SupplierID from Suppliers where Country = 'Peru');

