--
--Unidad III
--

use Northwind
go

--
--Consultas básicas
--

--
--Consultas básicas - Select (1)
--

select ProductID, ProductName, QuantityPerUnit
from Products

--
--Consultas básicas - Select (2)
--

select ProductName, QuantityPerUnit
from Products
where ProductID = 10

--
--Consultas básicas - Select (3)
--

select ProductID, ProductName, QuantityPerUnit
from Products
where QuantityPerUnit like '%boxes%'

--
--Consultas básicas - Select (4)
--

select ProductID, ProductName, QuantityPerUnit
from Products
where (QuantityPerUnit like '%boxes%') or (QuantityPerUnit like '%bottles%')

--
--Consultas básicas - Select (5)
--

select ProductID, ProductName, QuantityPerUnit
from Products
where ProductID between 1 and 10

--
--Consultas básicas - Select (6)
--

select LastName, FirstName, City
from Employees
where City in ('London', 'Seattle')

--
--Consultas básicas - Select (7)
--

select LastName, FirstName, Region
from Employees
where Region is not null

--
--Consultas básicas - Orden de datos
--

select ProductID, ProductName, UnitPrice
from Products
order by UnitPrice desc

--
--Consultas básicas - Eliminación de duplicados
--

select distinct CustomerID
from Orders
order by CustomerID

--
--Consultas básicas - Cambio de nombres a las columnas
--

select LastName 'Apellido', FirstName as 'Nombre'
from Employees

--
--Consultas básicas - Uso de constantes
--

select LastName 'Apellido', FirstName as 'Nombre', 'ID:'
from Employees

--
--Consultas básicas - CASE simple
--

select LastName, FirstName, 
	case TitleOfCourtesy
		when 'Mrs.' then 'Señora'
        when 'Mr.' then 'Señor'
        when 'Ms.' then 'Señorita'
        else 'Otro'
	end as 'Titulo'
from Employees

--
--Consultas básicas - CASE búsqueda
--

select ProductID, ProductName,
	case 
		when UnitPrice > 0 and UnitPrice < 30 then 'Hasta 50'
        when UnitPrice >= 30 and UnitPrice < 60 then 'Entre $30 y $60'
        else 'Más de $60'
	end as 'UnitPrice'
from Products

--
--Consultas básicas - Coalesce
--

select CustomerID, Region, Fax, coalesce(Region, Fax) as 'Not null'
from Customers

--
--Consultas básicas - isnull
--

select CustomerID, isnull(Region, '-') as 'Region'
from Customers

--también se puede usar coalesce

select CustomerID, coalesce(Region, '-') as 'Region'
from Customers
--
--Consultas básicas - nullif
--

create table Budgets (  
    dept int identity not null,  
    cur_year decimal null,  
    prev_year decimal null,
)

insert into Budgets (cur_year, prev_year) values (100000, 150000)
insert into Budgets (cur_year, prev_year) values (null, 300000)
insert into Budgets (cur_year, prev_year) values (0, 100000)
insert into Budgets (cur_year, prev_year) values (null, 150000)
insert into Budgets (cur_year, prev_year) values (300000, 250000)

select cur_year, prev_year
from Budgets

select cur_year, prev_year, coalesce(cur_year, prev_year) as 'Col1'
from Budgets

select cur_year, prev_year, coalesce(cur_year, prev_year) as 'Col1', nullif(coalesce(cur_year, prev_year), 0.00) as 'Col2'
from Budgets

select avg(nullif(coalesce(cur_year, prev_year), 0.00)) as 'Promedio'
from Budgets

drop table Budgets

--
--Consultas básicas - Inserción de filas con valores por defecto
--

create table Budgets (  
    dept int identity not null,  
    cur_year decimal null default 0,  
    prev_year decimal null default 0
)

insert into Budgets (cur_year, prev_year) values (100000, 150000)
insert into Budgets (cur_year, prev_year) values (null, 300000)
insert into Budgets (cur_year, prev_year) values (0, 100000)
insert into Budgets (cur_year, prev_year) values (null, 150000)
insert into Budgets (cur_year, prev_year) values (300000, 250000)
--valor por defecto en una columna
insert into Budgets (cur_year, prev_year) values (250000, default)

--valor por defecto en todas las columnas
insert into Budgets default values

select * from Budgets

drop table Budgets

--
--Consultas básicas - Borrado de filas
--

create table Budgets (  
    dept int identity not null,  
    cur_year decimal null,  
    prev_year decimal null
)

insert into Budgets (cur_year, prev_year) values (100000, 150000)
insert into Budgets (cur_year, prev_year) values (null, 300000)
insert into Budgets (cur_year, prev_year) values (0, 100000)
insert into Budgets (cur_year, prev_year) values (null, 150000)
insert into Budgets (cur_year, prev_year) values (300000, 250000)

select * from Budgets

delete from Budgets
where dept = 5

truncate table Budgets

drop table Budgets

--
--Consultas básicas - Modificación de filas
--

create table Budgets (  
    dept int identity not null,  
    cur_year decimal null,  
    prev_year decimal null
)

insert into Budgets (cur_year, prev_year) values (100000, 150000)
insert into Budgets (cur_year, prev_year) values (null, 300000)
insert into Budgets (cur_year, prev_year) values (0, 100000)
insert into Budgets (cur_year, prev_year) values (null, 150000)
insert into Budgets (cur_year, prev_year) values (300000, 250000)

select * from Budgets

update Budgets
set prev_year = 950000 where dept = 1

drop table Budgets

--
--Múltiples tablas
--


--
--Múltiples tablas - Inner join
--

select ProductID, ProductName, CategoryName
from Products join Categories
on Products.CategoryID = Categories.CategoryID

--
--Múltiples tablas - Outer join
--

select Customers.CustomerID, OrderID, OrderDate
from Customers left join Orders
on Customers.CustomerID = Orders.CustomerID

--
--Múltiples tablas - Cross join
--

select Employees.EmployeeID, OrderID
from Employees cross join Orders
order by Employees.EmployeeID, OrderID

select count(EmployeeID) from Employees
select count(OrderID) from Orders

--
--Múltiples tablas - Combinación de más de 2 tablas
--

select ProductID, ProductName, CompanyName, CategoryName
from Products join Suppliers
on Products.SupplierID = Suppliers.SupplierID
join Categories
on Products.CategoryID = Categories.CategoryID

--
--Múltiples tablas - Join de una tabla con sí misma
-- 





--
--Múltiples tablas - Combinación de varios ResultSets
-- 

select CompanyName, City, PostalCode 
from Customers
union all
select Name = (LastName + ' ' + FirstName), City, PostalCode 
from Employees

--
--Múltiples tablas - Creación de una tabla desde un ResultSet
-- 

select ProductName, UnitPrice
into PriceTable
from Products
go

select * from PriceTable

drop table PriceTable

--
--Múltiples tablas - Creación de una tabla temporal
-- 

select ProductName, UnitPrice
into #PriceTable
from Products

select * from #PriceTable

drop table #PriceTable

--
--Técnicas avanzadas - Subconsultas
-- 

select lastname, firstname
from Employees e
where exists (select * from orders where e.EmployeeID = orders.employeeid)

--
--Técnicas avanzadas - Inserción de filas basadas en otras tablas
-- 

insert into Customers
select SUBSTRING(firstname, 1, 3) + SUBSTRING(lastname, 1, 2), lastname, firstname, title, null, null, null, null, null, null, null
from employees

select * from Customers

delete from Customers
where [Address] is null and City is null and Region is null and PostalCode is null and Country is null and Phone is null and Fax is null

--
--Técnicas avanzadas - Borrado de filas basadas en otras tablas (delete con join)
-- 

delete from [Order Details]
from orders o join [Order Details] od
on o.OrderID = od.OrderID
where OrderDate = '4/14/2017'

--El primer from indica la tabla donde se borran las filas
--El segundo from hace referencia al criterio restrictivo (se pueden introducir joins y subconsultas)

--
--Técnicas avanzadas - Borrado de filas basadas en otras tablas (subconsultas)
-- 

delete from [Order Details]
where OrderID in (select OrderID from Orders where OrderDate = '4/14/2017')

--
--Técnicas avanzadas - Modificación de filas basadas en otras tablas (update con join)
-- 

update Products
set UnitPrice = UnitPrice + 2
from Products join Suppliers
on Products.SupplierID = Suppliers.SupplierID
where Suppliers.Country = 'Peru'


--
--Técnicas avanzadas - Modificación de filas basadas en otras tablas (subconsultas)
-- 

update Products
set UnitPrice = UnitPrice + 2
where SupplierID in (select SupplierID from Suppliers where Country = 'Peru')
