--
--Unidad IV
--

use Northwind
go

--
--Funciones de agrupamiento
--

--
--Funciones de agrupamiento (1)
--

select count(*) from Employees

--
--Funciones de agrupamiento (2)
--

select sum(quantity) as 'Cantidad de productos'
from [Order Details]
where OrderID = 10248

--
--Funciones de agrupamiento (3)
--

select count(*) as 'Cantidad de jefes'
from Employees
where ReportsTo is null

--
--Uso de GROUP BY
--

select ProductID, sum(Quantity) as 'Total'
from [Order Details]
group by ProductID

--
--Uso de GROUP BY con HAVING
--

select ProductID, sum(Quantity) as 'Total'
from [Order Details]
group by ProductID
having sum(Quantity) > 300

--
--Modificador ROLLUP (1)
--

select ProductID, sum(Quantity) as 'Total'
from [Order Details]
group by ProductID
with rollup
order by 2

--
--Modificador ROLLUP (2)
--

select ProductID, OrderID, sum(Quantity) as 'Total'
from [Order Details]
group by ProductID, OrderID
with rollup

--
--Modificador CUBE
--

select ProductID, OrderID, sum(Quantity) as 'Total'
from [Order Details]
group by ProductID, OrderID
with cube

--
--Función GROUPING
--

select ProductID, GROUPING(ProductID), OrderID, GROUPING(OrderID), sum(Quantity) as 'Total'
from [Order Details]
group by ProductID, OrderID
with cube

--
--Uso de TOP
--

select top 10 ProductID, sum(Quantity) as 'Total'
from [Order Details]
group by ProductID
order by 2 desc

select top 10 percent ProductID, sum(Quantity) as 'Total'
from [Order Details]
group by ProductID
order by 2 desc

--
--Uso de TOP WITH TIES
--

select top 10 with ties ProductID, sum(Quantity) as 'Total'
from [Order Details]
group by ProductID
order by 2 desc
go

--
--Creación de vistas
--

create view V_Orders (ID, Fecha, Cliente, Producto, Precio, Cantidad)
as 
	select Orders.OrderID, OrderDate, CompanyName, ProductName, [Order Details].UnitPrice, Quantity
	from Orders join Customers
	on Orders.CustomerID = Customers.CustomerID
	join [Order Details]
	on Orders.OrderID = [Order Details].OrderID
	join Products
	on [Order Details].ProductID = Products.ProductID
go

select * from V_Orders
go

drop view V_Orders
go

--
--Modificación de vistas
--

alter view V_Orders (ID, Fecha, Cliente, Producto, Cantidad) 
as --se saca el precio
	select Orders.OrderID, OrderDate, CompanyName, ProductName, Quantity
	from Orders join Customers
	on Orders.CustomerID = Customers.CustomerID
	join [Order Details]
	on Orders.OrderID = [Order Details].OrderID
	join Products
	on [Order Details].ProductID = Products.ProductID
go

select * from V_Orders

--
--Encriptación de vistas
--

declare @idvista int --variable para guardar el identificador de la vista
set @idvista = OBJECT_ID('V_Orders')

select id, text from sys.syscomments 
where id = @idvista
go

exec sp_helptext 'V_Orders'
go

alter view V_Orders (ID, Fecha, Cliente, Producto, Cantidad) 
--with encryption
as --se saca el precio
	select Orders.OrderID, OrderDate, CompanyName, ProductName, Quantity
	from Orders join Customers
	on Orders.CustomerID = Customers.CustomerID
	join [Order Details]
	on Orders.OrderID = [Order Details].OrderID
	join Products
	on [Order Details].ProductID = Products.ProductID
go


--
--Borrado de vistas
--

drop view V_Orders
go


--
--Cadena de permisos
--

CREATE LOGIN [WIN7\Lucia] FROM WINDOWS WITH DEFAULT_DATABASE=[Northwind], DEFAULT_LANGUAGE=[Español]
GO

CREATE LOGIN [WIN7\Maria] FROM WINDOWS WITH DEFAULT_DATABASE=[Northwind], DEFAULT_LANGUAGE=[Español]
GO

CREATE LOGIN [WIN7\Pedro] FROM WINDOWS WITH DEFAULT_DATABASE=[Northwind], DEFAULT_LANGUAGE=[Español]
GO

CREATE USER [Lucia] FOR LOGIN [WIN7\Lucia] WITH DEFAULT_SCHEMA=[dbo]
GO

CREATE USER [Maria] FOR LOGIN [WIN7\Maria] WITH DEFAULT_SCHEMA=[dbo]
GO

CREATE USER [Pedro] FOR LOGIN [WIN7\Pedro] WITH DEFAULT_SCHEMA=[dbo]
GO

CREATE TABLE dbo.Tabla1(
	col1 int NULL,
	col2 varchar(50) NULL
) ON [PRIMARY]
GO

alter authorization on dbo.Tabla1 to Lucia

select USER_NAME(principal_id)
from sys.tables 
where name = 'Tabla1'
go

create view Vista1 as
select * from Tabla1
go

alter authorization on dbo.Vista1 to Lucia

select USER_NAME(principal_id)
from sys.views
where name = 'Vista1'
go

create view Vista2 as
select * from Vista1
go

alter authorization on dbo.Vista2 to Lucia

select USER_NAME(principal_id)
from sys.views
where name = 'Vista2'
go

grant select on object::dbo.Tabla1 to Pedro
grant select on object::dbo.Vista1 to Pedro
grant select on object::dbo.Vista2 to Pedro

revoke select on object::dbo.Tabla1 to Pedro
revoke select on object::dbo.Vista1 to Pedro
revoke select on object::dbo.Vista2 to Pedro








--
--Información sobre las vistas
--

select * from INFORMATION_SCHEMA.TABLES
where TABLE_NAME = 'V_Orders'
--nombre de las vistas

select * from INFORMATION_SCHEMA.VIEW_TABLE_USAGE
where VIEW_NAME = 'V_Orders'
--objetos que usan las vistas

select * from INFORMATION_SCHEMA.VIEWS
where TABLE_NAME = 'V_Orders'
--definición de las vistas

select * from INFORMATION_SCHEMA.VIEW_COLUMN_USAGE
where VIEW_NAME = 'V_Orders'
--columnas uasadas por las vistas

select * from sys.views
--vistas de la BD

exec sp_helptext 'V_Orders'
--definición de la vista

exec sp_depends 'V_Orders'
--dependencias de la vista


